#include "comdef.h"
#include "int-flash.h"

/* Chip-specific internal flash API header files */
#if defined(__TMS470__)
  #include "..\f05api\f05.h"
  #include "..\f05api\flash470.h"
#endif


uint16 perform_int_flash_program()
{
#if defined(__TMS470__)
  uint16 bank;
  uint16 i;
  uint16 sector;
  uint32 sector_addr, addr;
  FLASH_STATUS_ST stat;

  /* Address must be aligned on 4-byte boundary */
  if ( address & 3 ) {
    status = STATUS_ADDRESS_ALIGN;
    return 0; /* return ERROR */
  }
  if ( length & 3 ) {
    status = STATUS_LENGTH_ALIGN;
    return 0; /* return ERROR */
  }

  /* Check if non-FFFFFFFF keys (both, MSM & flash write protection)
     programming requested and not enabled */

  if (fProgKeys != 0x12345678) {
    addr = SECTOR_SIZE(sector_def[0]) - 0x20;
    for (i=0; i<8; i++) { /* 8 32-bit KEYS */
	  if ((address <= addr) && ((address + length) > addr))
	    if (0xFFFFFFFF != buffer[(addr-address)>>2]) {
          status = STATUS_KEYS_REPROGRAMMING;
          return 0; /* return ERROR */
		}
	  addr += 4;
	}
  }

  /* find sector & bank for address */
  for (sector=0; sector < sectors; sector++) {
    if (address < (sector_addr + SECTOR_SIZE(sector_def[sector]))) {
      /* sector found */
	  bank = FLASH_BANK(sector_def[sector]);
	  break;
	}
	sector_addr += SECTOR_SIZE(sector_def[sector]);
  }

  if (sector >= sectors) {
    status = STATUS_INVALID_PARAMETER;
    return 0; /* return ERROR */
  }

  /* Perform key matching */
  if (!Flash_Match_Key_B( (UINT32*)(base_address+SECTOR_SIZE(sector_def[0]) - 0x10),
                          (UINT32*)&flsh_passwd[0],
   				          (FLASH_ARRAY_ST) ctrlr_base)) {

    status = STATUS_FLASH_KEY_MISMATCH;
    return 0; /* return ERROR */
  }

  if (!Flash_Prog_B( (UINT32*)(address + base_address),
                     (UINT32*)buffer,
					 length>>2,
                     (FLASH_CORE) bank,
				     5,
				     (FLASH_ARRAY_ST) ctrlr_base,
				     &stat) ) {

    status = STATUS_FLASH_PROGRAM_FAIL;
    return 0; /* return ERROR */
  }

  if (!Flash_Verify_B( (UINT32*)(address + base_address),
                     (UINT32*)buffer,
					 length>>2,
                     (FLASH_CORE) bank,
				     (FLASH_ARRAY_ST) ctrlr_base,
				     &stat) ) {

    status = STATUS_VERIFY_ERR;
    return 0; /* return ERROR */
  }

  address += length;
             
  return 1;
#else
  status = STATUS_NOT_IMPLEMENTED;
  return 0; /* return ERROR */
#endif
}

uint16 perform_int_flash_erase()
{
#if defined(__TMS470__)
  uint16 sector;
  uint32 sector_addr = 0;
  uint16 bank;
  uint32 size;
  FLASH_STATUS_ST stat;

  // <address in sector> to address of begin of sector, sector# and bank#
  for (sector=0; sector < sectors; sector++) {
    if (address < (sector_addr + SECTOR_SIZE(sector_def[sector]))) {
      /* sector found */
	  bank = FLASH_BANK(sector_def[sector]);
	  size = SECTOR_SIZE(sector_def[sector]);
	  break;
	}
	sector_addr += SECTOR_SIZE(sector_def[sector]);
  }

  if (sector >= sectors) {
    status = STATUS_INVALID_PARAMETER;
    return 0; /* return ERROR */
  }

  /* Perform key matching */
  if (!Flash_Match_Key_B( (UINT32*)(base_address+SECTOR_SIZE(sector_def[0]) - 0x10),
                          (UINT32*)&flsh_passwd[0],
   				          (FLASH_ARRAY_ST) ctrlr_base)) {

    status = STATUS_FLASH_KEY_MISMATCH;
    return 0; /* return ERROR */
  }

  /* Perform sector "compaction" - normalize depleted bits */

  if (!Flash_Compact_B( (UINT32*)(sector_addr + base_address),
                      (FLASH_CORE) bank,
		   		      (FLASH_SECT) sector,
				      5,
				      (FLASH_ARRAY_ST) ctrlr_base,
				      &stat)) {
    status = STATUS_FLASH_COMPACTING_FAIL;
    return 0; /* return ERROR */
  } 

  /* Test sector for already blank, and if yes - disable preconditioning */

  if (Flash_Blank_B( (UINT32*)(sector_addr + base_address),
                     size/4,
					 (FLASH_CORE) bank,
                     (FLASH_ARRAY_ST) ctrlr_base,
					 &stat)) {
	stat.stat1 = 0x12345678;
  } else {
	stat.stat1 = 0;
  }

  /* Erase this sector */

  if (!Flash_Erase_B( (UINT32*)(sector_addr + base_address),
                      size/4,
                      (FLASH_CORE) bank,
 		   		      (FLASH_SECT) sector,
				      5,
				      (FLASH_ARRAY_ST) ctrlr_base,
					 &stat)) {

    status = STATUS_FLASH_ERASE_FAIL;
    return 0; /* return ERROR */
  } 

  return 1;
#else
  status = STATUS_NOT_IMPLEMENTED;
  return 0; /* return ERROR */
#endif
}

